<?php

namespace Drupal\gnode_request;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\gnode_request\Plugin\Group\Relation\GroupNodeRequest;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupRelationship;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\node\NodeInterface;
use Drupal\state_machine\Plugin\Field\FieldType\StateItemInterface;

/**
 * Membership Request Manager class.
 */
class GnodeRequestManager {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * PrivacyManager constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
  }

  /**
   * Get group node request.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   Group.
   * @param \Drupal\node\NodeInterface $node
   *   Node.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface|null
   *   Group relationship or NULL.
   */
  public function getRequest(GroupInterface $group, NodeInterface $node) {
    $group_type = $group->getGroupType();
    if (!$group_type->hasPlugin('group_node_request:' . $node->bundle())) {
      return NULL;
    }

    $group_node_requests = $group->getRelationshipsByEntity($node, 'group_node_request:' . $node->bundle());
    if (!empty($group_node_requests)) {
      return reset($group_node_requests);
    }

    return NULL;
  }

  /**
   * Approve a membership request.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $group_relationship
   *   Group membership request group relationship.
   *
   * @return bool
   *   Result.
   */
  public function approve(GroupRelationshipInterface $group_relationship) {
    $this->updateStatus($group_relationship, GroupNodeRequest::TRANSITION_APPROVE);
    return $group_relationship->save() == SAVED_UPDATED;
  }

  /**
   * Reject a membership request.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $group_relationship
   *   Group membership request group relationship.
   *
   * @return bool
   *   Result.
   */
  public function reject(GroupRelationshipInterface $group_relationship) {
    $this->updateStatus($group_relationship, GroupNodeRequest::TRANSITION_REJECT);
    return $group_relationship->save() == SAVED_UPDATED;
  }

  /**
   * Update status of a membership request.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $group_relationship
   *   Group membership request group relationship.
   * @param string $transition_id
   *   Transition approve | reject.
   *
   * @throws \InvalidArgumentException
   *   If workflow transition does not exist, or isn't allowed.
   * @throws \Drupal\gnode_request\GnodeRequestManagerException
   *   When called for a relationsip other than group_node_request.
   */
  public function updateStatus(GroupRelationshipInterface $group_relationship, $transition_id) {
    if ($group_relationship->getPlugin()->getBaseId() != 'group_node_request') {
      throw new GnodeRequestManagerException('Only group relationship of "Group node request" is allowed.');
    }
    $state_item = $group_relationship->get(GroupNodeRequest::STATUS_FIELD)->first();
    assert($state_item instanceof StateItemInterface);
    $state_item->applyTransitionById($transition_id);
    $group_relationship->set('gnode_request_updated_by', $this->currentUser->id());
  }

  /**
   * Create group membership request group relationship.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   Group.
   * @param \Drupal\node\NodeInterface $node
   *   Node.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface
   *   Group membership request group relationship.
   */
  public function create(GroupInterface $group, NodeInterface $node) {
    $group_type = $group->getGroupType();
    if (!$group_type->hasPlugin('group_node_request:' . $node->getType())) {
      return NULL;
    }

    if ($group->getRelationshipsByEntity($node, 'group_node:' . $node->getType())) {
      throw new GnodeRequestManagerException('This node is already in the group');
    }

    $plugin_id = 'group_node_request:' . $node->getType();
    $relationship_type_storage = $this->entityTypeManager->getStorage('group_relationship_type');

    $group_relationship = GroupRelationship::create([
      'type' => $relationship_type_storage->getRelationshipTypeId($group_type->id(), $plugin_id),
      'gid' => $group->id(),
      'entity_id' => $node->id(),
      GroupNodeRequest::STATUS_FIELD => GroupNodeRequest::REQUEST_NEW,
    ]);

    // We have to set transition here. Once the group_relationship saved events will
    // be correctly fired.
    $this->updateStatus($group_relationship, GroupNodeRequest::TRANSITION_CREATE);

    return $group_relationship;
  }

}
