<?php

namespace Drupal\Tests\gnode_request\Functional;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\gnode_request\Plugin\Group\Relation\GroupNodeRequest;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\group\GroupMembership;
use Drupal\group\PermissionScopeInterface;
use Drupal\Tests\group\Functional\GroupBrowserTestBase;
use Drupal\state_machine\Plugin\Field\FieldType\StateItemInterface;
use Drupal\user\RoleInterface;

/**
 * Tests the behavior of the group type form.
 *
 * @group group
 */
class GroupNodeRequestFormTest extends GroupBrowserTestBase {

  use StringTranslationTrait;
  use ContentTypeCreationTrait;
  use NodeCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'group',
    'group_test_config',
    'node',
    'state_machine',
    'gnode',
    'gnode_request',
    'views',
  ];

  /**
   * Membership request manager.
   *
   * @var \Drupal\gnode_request\GnodeRequestManager
   */
  protected $gnodeRequestManager;

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $group;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->group = $this->createGroup(['type' => 'default']);
    $this->group2 = $this->createGroup(['type' => 'default']);
    $group_type = $this->group->getGroupType();
    $this->groupContentType = $this->createContentType(['type' => 'page']);
    $this->otherContentType = $this->createContentType();

    $plugin_id = 'group_node_request:page';
    $this->gnodeRequestManager = $this->container->get('gnode_request.gnode_request_manager');

    // Install group_node_request group relationship.

    /** @var \Drupal\group\Entity\Storage\GroupRelationshipTypeStorageInterface $group_relationship_type_storage */
    $group_relationship_type_storage = $this->entityTypeManager->getStorage('group_relationship_type');
    $config = [
      'group_cardinality' => 0,
      'entity_cardinality' => 1,
    ];
    $group_relationship_type_storage->createFromPlugin($group_type, 'group_node:page', $config)->save();

    $config = [
      'group_cardinality' => 0,
      'entity_cardinality' => 1,
      'remove_group_node_request' => FALSE,
    ];
    $group_relationship_type_storage->createFromPlugin($group_type, $plugin_id, $config)->save();

    // Add a text field to the group relationship type.
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_test_text',
      'entity_type' => 'group_relationship',
      'type' => 'text',
    ]);
    $field_storage->save();

    $relation_type_id = $group_relationship_type_storage->getRelationshipTypeId($group_type->id(), $plugin_id);
    FieldConfig::create([
      'field_storage' => $field_storage,
      'bundle' => $relation_type_id,
      'label' => 'String long',
    ])->save();

    EntityFormDisplay::create([
      'targetEntityType' => 'group_relationship',
      'bundle' => $relation_type_id,
      'mode' => 'default',
      'status' => TRUE,
    ])->setComponent('field_test_text', ['type' => 'text_textfield'])->enable()->save();

    // Add permissions to members.
    $this->createGroupRole([
      'group_type' => $group_type->id(),
      'scope' => PermissionScopeInterface::INSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => [
        'administer group node requests',
      ],
    ]);

    // Allow outsider to make group node request.
    $this->createGroupRole([
      'group_type' => $group_type->id(),
      'scope' => PermissionScopeInterface::OUTSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => ['request node membership'],
    ]);
  }

  /**
   * Tests approval form.
   */
  public function testApprovalForm() {
    $node = $this->createNode(['type' => 'page']);
    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $group_node_request->save();

    $this->drupalGet("/group/{$this->group->id()}/content/{$group_node_request->id()}/approve-node");
    $this->assertSession()->statusCodeEquals(403);

    $manager_account = $this->createUser();
    $this->group->addMember($manager_account);
    $this->group2->addMember($manager_account);
    $this->drupalLogin($manager_account);

    $this->drupalGet("/group/{$this->group2->id()}/content/{$group_node_request->id()}/approve-node");
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet("/group/{$this->group->id()}/content/{$group_node_request->id()}/approve-node");
    $this->assertSession()->statusCodeEquals(200);

    $submit_button = 'Approve';
    $this->assertSession()->buttonExists($submit_button);
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->linkByHrefExists($this->group->toUrl()->toString());
    $this->assertSession()->pageTextContains(strip_tags($this->t('Are you sure you want to approve a request for %node?', ['%node' => $node->label()])->render()));

    $this->submitForm([], $submit_button);
    $this->assertSession()->pageTextContains($this->t('Request approved'));

    $group_memberships = $this->group->getRelationshipsByEntity($node, 'group_node:page');
    $this->assertCount(1, $group_memberships);
    $group_requests = $this->group->getRelationshipsByEntity($node, 'group_node_request:page');
    $this->assertCount(1, $group_memberships);
    $group_request = reset($group_requests);
    $state_item = $group_request->get(GroupNodeRequest::STATUS_FIELD)->first();
    assert($state_item instanceof StateItemInterface);
    $this->assertEquals(GroupNodeRequest::REQUEST_APPROVED, $state_item->value);
  }

  /**
   * Tests reject form.
   */
  public function testRejectForm() {
    $node = $this->createNode(['type' => 'page']);
    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $group_node_request->save();

    $this->drupalGet("/group/{$this->group->id()}/content/{$group_node_request->id()}/reject-node");
    $this->assertSession()->statusCodeEquals(403);

    $manager_account = $this->createUser();
    $this->group->addMember($manager_account);
    $this->drupalLogin($manager_account);

    $this->drupalGet("/group/{$this->group->id()}/content/{$group_node_request->id()}/reject-node");
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet("/group/{$this->group->id()}/content/{$group_node_request->id()}/reject-node");
    $this->assertSession()->statusCodeEquals(200);

    $submit_button = 'Reject';
    $this->assertSession()->buttonExists($submit_button);
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->linkByHrefExists($this->group->toUrl()->toString());
    $this->assertSession()->pageTextContains(strip_tags($this->t('Are you sure you want to reject a request for %node?', ['%node' => $node->label()])->render()));

    $this->submitForm([], $submit_button);
    $this->assertSession()->pageTextContains($this->t('Request rejected'));

    $group_memberships = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertEmpty($group_memberships, 'Group membership was not found.');
    $group_requests = $this->group->getRelationshipsByEntity($node, 'group_node_request:' . $node->bundle());
    $this->assertCount(1, $group_requests, 'Request exists.');
    $group_request = reset($group_requests);
    $state_item = $group_request->get(GroupNodeRequest::STATUS_FIELD)->first();
    assert($state_item instanceof StateItemInterface);
    $this->assertEquals(GroupNodeRequest::REQUEST_REJECTED, $state_item->value);
  }

  /**
   * Tests request form.
   *
   * @todo this should by default be the create from supplied by group.
   *
  public function testRequestForm() {
    // Access request form as a member.
    $this->drupalGet("/group/{$this->group->id()}/request-node");
    $this->assertSession()->statusCodeEquals(403);

    // Access request form as a not member.
    $account = $this->createUser();
    $this->drupalLogin($account);

    $this->drupalGet("/group/{$this->group->id()}/request-node");
    $this->assertSession()->statusCodeEquals(200);

    $submit_button = 'Request group membership';
    $this->assertSession()->buttonExists($submit_button);
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->linkByHrefExists($this->group->toUrl()->toString());
    $this->assertSession()->pageTextContains(strip_tags($this->t('Request membership for group %group', ['%group' => $this->group->label()])->render()));
    $this->assertSession()->fieldExists('field_test_text[0][value]');
    $membership_request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNull($membership_request);

    $this->submitForm([], $submit_button);
    $this->assertSession()->pageTextContains($this->t('Your request is waiting for approval'));

    $membership_request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertTrue($membership_request instanceof GroupRelationshipInterface, 'Membership request has been successfully created.');

    $this->drupalGet("/group/{$this->group->id()}/request-node");
    $this->assertSession()->statusCodeEquals(403);

  }
  */
}
