<?php

namespace Drupal\Tests\gnode_request\Kernel;

use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\gnode_request\Plugin\Group\Relation\GroupNodeRequest;
use Drupal\group\Entity\GroupRelationship;
use Drupal\group\PermissionScopeInterface;
use Drupal\Tests\group\Kernel\GroupKernelTestBase;
use Drupal\node\NodeInterface;

/**
 * Tests the general behavior of group entities.
 *
 * @coversDefaultClass \Drupal\group\Entity\Group
 * @group group
 */
class GroupNodeRequestTest extends GroupKernelTestBase {

  use ContentTypeCreationTrait;
  use NodeCreationTrait;

  /**
   * Membership request manager.
   *
   * @var \Drupal\gnode_request\GnodeRequestManager
   */
  protected $gnodeRequestManager;

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $group;

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $groupRelationshipTypeStorage;

  /**
   * The group relationship type for group membership request.
   *
   * @var \Drupal\group\Entity\GroupRelationshipTypeInterface
   */
  protected $groupRelationshipType;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'gnode',
    'gnode_request',
    'state_machine',
    'node',
    'field',
    'system',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('node');
    $this->installSchema('node', 'node_access');

    $this->installConfig([
      'gnode',
      'gnode_request',
      'state_machine',
      'node',
      'filter',
    ]);

    $this->gnodeRequestManager = $this->container->get('gnode_request.gnode_request_manager');

    $group_type = $this->createGroupType();
    $this->group = $this->createGroup(['type' => $group_type->id()]);
    // Enable group membership request group relationship plugin.
    $config = [
      'group_cardinality' => 0,
      'entity_cardinality' => 1,
      'remove_group_node_request' => FALSE,
    ];

    $this->groupContentType = $this->createContentType(['type' => 'page']);
    $this->otherContentType = $this->createContentType();

    $this->groupRelationshipTypeStorage = $this->entityTypeManager->getStorage('group_relationship_type');
    $this->groupRelationshipType = $this->groupRelationshipTypeStorage->createFromPlugin($group_type, 'group_node_request:page', $config);
    $this->groupRelationshipType->save();
    $this->groupRelationshipTypeStorage->createFromPlugin($group_type, 'group_node:page')->save();
  }

  /**
   * Test the creation of the request when node is already in the group.
   */
  public function testAddRequestForExistingContent() {
    $node = $this->createNode();
    $this->group->addRelationship($node, 'group_node:page');

    $this->expectExceptionMessage('This node is already in the group');
    $this->gnodeRequestManager->create($this->group, $node);
  }

  /**
   * Test validation of the request when node already is a member.
   */
  public function testRequestWithMemberAsContent() {
    $node = $this->createNode();

    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $this->group->addRelationship($node, 'group_node:page');

    $violations = $group_node_request->validate();
    $this->assertNotEmpty($violations);

    $messages = [];
    foreach ($violations as $violation) {
      $messages[] = $violation->getMessage()->getUntranslatedString();
    }

    $this->assertTrue(in_array('"%name" is already in the group', $messages), 'The validation of group membership request is found.');
  }

  /**
   * Test approval.
   */
  public function testRequestApproval() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    // Check group relations.
    $group_relations = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertEmpty($group_relations);

    // Approve.
    $this->gnodeRequestManager->approve($group_node_request);

    // Check status.
    $this->assertEquals($group_node_request->gnode_request_status->value, GroupNodeRequest::REQUEST_APPROVED);
    // Check group relations.
    $group_relations = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertCount(1, $group_relations);
  }

  /**
   * Test rejection.
   */
  public function testRequestRejection() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    // Check group relations.
    $group_relations = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertEmpty($group_relations);

    // Reject.
    $this->gnodeRequestManager->reject($group_node_request);

    // Check status.
    $this->assertEquals($group_node_request->gnode_request_status->value, GroupNodeRequest::REQUEST_REJECTED);
    // Check group relations.
    $group_relations = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertEmpty($group_relations);
  }

  /**
   * Test wrong status update workflow.
   */
  public function testWrongRequestWorkflow() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    $this->gnodeRequestManager->approve($group_node_request);
    $this->assertEquals($group_node_request->get(GroupNodeRequest::STATUS_FIELD)->value, GroupNodeRequest::REQUEST_APPROVED);
    $this->expectExceptionMessage('The transition "reject" is currently not allowed. (Current state: "approved".)');
    $this->gnodeRequestManager->reject($group_node_request);
  }

  /**
   * Test deletion of group request after node deletion.
   */
  public function testNodeDeletion() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    $request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertEquals($request->id(), $group_node_request->id());

    $node->delete();

    $request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNull($request);
  }

  /**
   * Test deletion of group request after group node deletion.
   */
  public function testGroupNodeDeletion() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    $this->gnodeRequestManager->approve($group_node_request);

    $group_nodes = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertCount(1, $group_nodes);
    $group_node = reset($group_nodes);
    $group_node->delete();

    $request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNull($request);
  }

  /**
   * Test the request is removed when group node is removed.
   */
  public function testRequestRemovalWhenGroupNodeRemoved() {
    $node = $this->createNode();
    $group_node_request = $this->createRequest($node);

    $this->group->addRelationship($node, 'group_node:page');

    $group_nodes = $this->group->getRelationshipsByEntity($node, 'group_node:' . $node->bundle());
    $this->assertCount(1, $group_nodes);
    $group_node = reset($group_nodes);
    $group_node->delete();

    $request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNull($request);
  }

  /**
   * Test wrong group relationship.
   */
  public function testWrongGroupRelationship() {
    $node = $this->createNode();
    $relation_type_id = $this->groupRelationshipTypeStorage->getRelationshipTypeId($this->group->getGroupType()->id(), 'group_node:page');
    $group_membership_group_relationship = GroupRelationship::create([
      'type' => $relation_type_id,
      'gid' => $this->group->id(),
      'entity_id' => $node->id(),
    ]);

    $this->expectExceptionMessage('Only group relationship of "Group node request" is allowed.');
    $this->gnodeRequestManager->approve($group_membership_group_relationship);
  }

  /**
   * Test group membership removal with disabled settings.
   */
  public function testRequestRemovalWithDisabledSettings() {
    $node = $this->createNode();

    // Add first group membership request.
    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $group_node_request->save();

    // Add the node to the group.
    $this->group->addRelationship($node, 'group_node:' . $node->bundle());

    // Since removal is disabled we should see find group membership request.
    $group_node_request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNotNull($group_node_request);
  }

  /**
   * Test group membership removal with enabled settings.
   */
  public function testRequestRemovalWithEnabledSettings() {
    $config = [
      'group_cardinality' => 0,
      'entity_cardinality' => 1,
      'remove_group_node_request' => TRUE,
    ];
    $this->groupRelationshipType->updatePlugin($config);
    $node = $this->createNode();

    // Add first group membership request.
    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $group_node_request->save();

    // Add the node to the group.
    $this->group->addRelationship($node, 'group_node:' . $node->bundle());

    // Since removal is disabled we should see find group membership request.
    $group_node_request = $this->gnodeRequestManager->getRequest($this->group, $node);
    $this->assertNull($group_node_request);
  }

  /**
   * Create group node request.
   *
   * @return \Drupal\group\Entity\GroupRelationshipInterface
   *   Group node request.
   */
  public function createRequest(NodeInterface $node) {
    $group_node_request = $this->gnodeRequestManager->create($this->group, $node);
    $group_node_request->save();

    $this->assertEquals($group_node_request->get(GroupNodeRequest::STATUS_FIELD)->value, GroupNodeRequest::REQUEST_PENDING);

    return $group_node_request;
  }

}
